@@ -1,5 +1,3 @@
-#[cfg(feature = "abomonation-serialize")]
-mod abomonation;
 mod blas;
 mod cg;
 mod conversion;
@@ -16,3 +14,6 @@ mod matrixcompare;
 
 #[cfg(feature = "arbitrary")]
 pub mod helper;
+
+#[cfg(feature = "macros")]
+mod macros;
